{-# OPTIONS_GHC -Wno-overlapping-patterns #-}
module Main where

import LSystemComp
import Control.Concurrent
import Data.List

-- Generate the tree fractal
treeFidget :: Int -> Command
treeFidget x = f x :#: Turn 90 :#: f x :#: Turn 90 :#: f x :#: Turn 90 :#: f x
    where
        f 0 = Go 10
        f x = f (x-1) :#: f (x-1) :#: p :#: Branch (p :#: f (x-1) :#: n :#: f (x-1) :#: n :#: f (x-1)) :#: n :#: Branch (n :#: f (x-1) :#: p :#: f (x-1) :#: p :#: f (x-1))
        n   = Turn (-22.5)
        p   = Turn 22.5

-- rotate the tree by a given angle
rotateTree :: (Int -> Command) -> Int  -> Float -> Command
rotateTree f x  a = Turn (-a) :#: f x

-- change the colour of the Pen based on the position/iteration in the list
changeColour :: [Command] -> [Command]
changeColour fs = [if remFind f == Just 1 then GrabPen black :#: f
                    else if remFind f == Just 2 then GrabPen blue :#: f
                    else if remFind f == Just 0 then GrabPen red :#: f
                    else GrabPen green :#: f | f <- fs]
            where
                remFind f = (`mod` 3) <$> elemIndex f fs 

-- IO function to generate angles, then a list of commands, and displaying the fractals
generateAndRotate :: Int -> Int -> IO ()
generateAndRotate numIterations delayMillis = do
    let angles = map fromIntegral [30 .. (numIterations - 1)*30] :: [Float]
    let outputList = changeColour [rotateTree treeFidget 4 a| a <- angles]
    let indices = [0..length outputList-1]
    mapM_ (\i -> do
        let outputNo = outputList !! i
        display outputNo
        threadDelay (delayMillis * 1000)
      ) indices

-- main function with example inputs
main :: IO ()
main = generateAndRotate 20 200